/* Easy Machine: assember driver.
   This is a derivative work based on Jitter, which is released under
   the GNU General Public License, version 3 or later. */

/* Use Gnulib. */
# include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdbool.h>

#include <argp.h>

#include <easy-machine/easy-machine-utility.h>
#include <easy-machine/easy-machine-machine.h>
#include <easy-machine/easy-machine-machine-interpreter.h>

#include <easy-machine/easy-machine-assembler.h>
#include <easy-machine/easy-machine-asm-parser.h>


/* Command line handling.
 * ************************************************************************** */

void
easy_machine_assmbler_command_line_initialize (struct easy_machine_assmbler_command_line *cl)
{
  cl->dry_run = false;
  cl->debug = false;
  cl->dump = false;
  cl->dump_from_address = -1;
  cl->dump_to_address = -1;
}

/* Update our option state with the information from a single command-line
   option. */
static error_t
parse_opt (int key, char *arg, struct argp_state *state)
{
  struct easy_machine_assmbler_command_line *cl = state->input;
  switch (key)
    {
      /* FIXME: within the case I could use state->arg_num to get the current
         non-option argument index, if needed. */
    case ARGP_KEY_INIT:
      /* Do nothing. */
      break;
    case 'r':
      cl->dry_run = true;
      break;
    case 'd':
      cl->debug = true;
      break;
    case 'D':
      cl->dump = true;
      break;
    case 'f':
      cl->dump_from_address = easy_machine_scan_literal (arg);
      cl->dump = true;
      break;
    case 't':
      cl->dump_to_address = easy_machine_scan_literal (arg);
      cl->dump = true;
      break;

    case ARGP_KEY_ARG:
      cl->input_pathname = arg;
      break;

    /* case ARGP_KEY_NO_ARGS: /\* If this case is omitted, the default is sensible. *\/ */
    /*   argp_error (state, "you have to supply the input file\n"); */
    /*   break; */

    /* case ARGP_KEY_SUCCESS: /\* If this case is omitted, the default is sensible. *\/ */
    /*   break; */

    case ARGP_KEY_END:
      if (state->arg_num != 1)
        argp_error (state, "you gave %i input files instead of one.",
                    (int)state->arg_num);
      break;

    /* case ARGP_KEY_FINI: /\* If this case is omitted, the default is sensible. *\/ */
    /*   break; */

    default:
      return ARGP_ERR_UNKNOWN;
    }
  return 0;
}

/* Command-line option specification. */
static struct argp_option main_option_specification[] =
  {/* Most frequently used options. */
   {NULL, '\0', NULL, OPTION_DOC, "Frequently used options:", 10},
   {"dry-run", 'r', NULL, 0,
    "Do not run the program"},
   {"debug", 'd', NULL, 0,
    "Print out debugging information"},
   {"dump", 'D', NULL, 0,
    "Dump memory (without specifying --dump-from/-f and --dump-to/-t, using default bounds)"},
   {"dump-from", 'f', "ADDRESS", 0,
    "Dump memory starting from the given address, included"},
   {"dump-to", 't', "ADDRESS", 0,
    "Dump memory up to the given address, not included"},

   /* Common GNU-style options. */
   {NULL, '\0', NULL, OPTION_DOC, "Common GNU-style options:", -1},
   /* These are automatically generated. */

   /* Option terminator. */
   { 0 }};

const char *argp_program_version
  = "Easy Machine assembler/simulator (" EASY_MACHINE_PACKAGE_NAME ") "
    EASY_MACHINE_PACKAGE_VERSION "\n"
    "Copyright (C) 2016, 2017, 2019 Luca Saiu.\n"
    "This is a derivative work based on Jitter, which is released under\n"
    "the GNU General Public License, version 3 or later.\n"
    "Jitter comes with ABSOLUTELY NO WARRANTY.\n"
    "You may redistribute copies of Jitter under the terms of the\n"
    "GNU General Public License, version 3 or any later version published by\n"
    "the Free Software Foundation.  For more information see the file named\n"
    "COPYING in the source distribution or the git repository.\n\n"
    "Written by Luca Saiu <http://ageinghacker.net> (Jitter, its runtime, "
    "this derivative program).";
const char *argp_program_bug_address = EASY_MACHINE_PACKAGE_BUGREPORT;

/* The parser main data structure. */
static struct argp argp =
  {
    main_option_specification,
    parse_opt,
    NULL,
    "Assemble an Easy Machine assembly file into memory and run it."
  };




/* Main.
 * ************************************************************************** */

int
main (int argc, char **argv)
{
  /* Fill a command-line struct with defaults. */
  struct easy_machine_assmbler_command_line cl;
  easy_machine_assmbler_command_line_initialize (& cl);

  /* Parse our arguments, overwriting some of * cl with what the user
     demands.  */
  argp_parse (&argp, argc, argv,
              0,//ARGP_IN_ORDER,
              0, & cl);

  /* Do the actual work. */
  easy_machine_initialize_global_state ();

  struct easy_machine_routine r;
  easy_machine_asm_initialize_routine (& r);
  easy_machine_parse_file (cl.input_pathname, & r);
  easy_machine_asm_load (& r, & cl);

  struct easy_machine_thread_state state;
  easy_machine_initialize_thread_state (& state,
                                        easy_machine_section_load_addresses [easy_machine_asm_section_code]);

  if (! cl.dry_run)
    easy_machine_run (& state, & cl);

  return EXIT_SUCCESS;
}
