/* Easy Machine.

   Machine interpreter: header. */

#ifndef EASY_MACHINE_MACHINE_H_
#define EASY_MACHINE_MACHINE_H_

#include <config.h>

#include <stdbool.h>
#include <stdint.h>


/* Type definitions.
 * ************************************************************************** */

typedef int32_t
easy_machine_word;

typedef uint32_t
easy_machine_uword;




/* Command line handling.
 * ************************************************************************** */

struct easy_machine_assmbler_command_line
{
  bool dry_run;
  bool debug;
  const char *input_pathname;
  bool dump;
  easy_machine_word dump_from_address, dump_to_address;
};




/* Machine global state.
 * ************************************************************************** */

/* The global state is implemented with globals in C.  This non-reentrancy
   should not be a problem in practice, as long as we don't need to run multiple
   machines in the same process. */
void
easy_machine_initialize_global_state (void);




/* Memory access.
 * ************************************************************************** */

uint8_t *
easy_machine_virtual_address_to_pointer (easy_machine_word address);

easy_machine_word
easy_machine_pointer_to_virtual_address (uint8_t *p);

uint8_t
easy_machine_load_byte (easy_machine_word address);

void
easy_machine_store_byte (easy_machine_word address,
                         uint8_t datum);

/* Convenience functions. */
easy_machine_word
easy_machine_load_word (easy_machine_word address);
void
easy_machine_store_word (easy_machine_word address,
                         easy_machine_word datum);




/* Machine thread state.
 * ************************************************************************** */

#define EASY_MACHINE_REGISTER_NO 256

typedef int
easy_machine_register_index;

struct easy_machine_thread_state
{
  easy_machine_word registers [EASY_MACHINE_REGISTER_NO];
  easy_machine_word vpc;
};

void
easy_machine_initialize_thread_state (struct easy_machine_thread_state *s,
                                      easy_machine_word pc);




/* Interpreter.
 * ************************************************************************** */

void
easy_machine_run (struct easy_machine_thread_state *s,
                  struct easy_machine_assmbler_command_line *cl);




/* Command-line options.
 * ************************************************************************** */

/* Command-line options.  Not used yet. */
struct easy_machine_machine_command_line
{
  bool debug;
};

void
easy_machine_machine_command_line_initialize
   (struct easy_machine_machine_command_line *cl);

#endif // #ifndef EASY_MACHINE_MACHINE_H_
